@@ -3,13 +3,17 @@ package Params::ValidationCompiler::Compiler;
 use strict;
 use warnings;
 
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
+use Carp qw( croak );
 use Eval::Closure qw( eval_closure );
+use List::Util 1.29 qw( pairkeys );
 use Params::ValidationCompiler::Exceptions;
 use Scalar::Util qw( blessed looks_like_number reftype );
 use overload ();
 
+our @CARP_NOT = ( 'Params::ValidationCompiler', __PACKAGE__ );
+
 BEGIN {
     ## no critic (Variables::RequireInitializationForLocalVars)
     local $@;
@@ -24,13 +28,14 @@ BEGIN {
 
     unless ($has_sub_util) {
         *set_subname = sub {
-            die
-                "Cannot name a generated validation subroutine. Please install Sub::Util.\n";
+            croak
+                'Cannot name a generated validation subroutine. Please install Sub::Util.';
         };
     }
 }
 
-my %known = map { $_ => 1 } qw( name name_is_optional params slurpy );
+my %known
+    = map { $_ => 1 } qw( name name_is_optional params slurpy named_to_list );
 
 # I'd rather use Moo here but I want to make things relatively high on the
 # CPAN river like DateTime use this distro, so reducing deps is important.
@@ -39,17 +44,33 @@ sub new {
     my %p     = @_;
 
     unless ( exists $p{params} ) {
-        die
-            qq{You must provide a "params" parameter when creating a parameter validator\n};
+        croak
+            q{You must provide a "params" parameter when creating a parameter validator};
     }
-    unless ( ref $p{params} eq 'HASH' || ref $p{params} eq 'ARRAY' ) {
+    if ( ref $p{params} eq 'HASH' ) {
+        croak q{The "params" hashref must contain at least one key-value pair}
+            unless %{ $p{params} };
+
+        croak
+            q{"named_to_list" must be used with arrayref params containing key-value pairs}
+            if $p{named_to_list};
+    }
+    elsif ( ref $p{params} eq 'ARRAY' ) {
+        croak q{The "params" arrayref must contain at least one element}
+            unless @{ $p{params} };
+    }
+    else {
         my $type
             = !defined $p{params} ? 'an undef'
             : ref $p{params} ? q{a } . ( lc ref $p{params} ) . q{ref}
             :                  'a scalar';
 
-        die
-            qq{The "params" parameter when creating a parameter validator must be a hashref or arrayref, you passed $type\n};
+        croak
+            qq{The "params" parameter when creating a parameter validator must be a hashref or arrayref, you passed $type};
+    }
+
+    if ( $p{named_to_list} && $p{slurpy} ) {
+        croak q{You cannot use "named_to_list" and "slurpy" together};
     }
 
     if ( exists $p{name} && ( !defined $p{name} || ref $p{name} ) ) {
@@ -58,14 +79,14 @@ sub new {
             ? 'an undef'
             : q{a } . ( lc ref $p{name} ) . q{ref};
 
-        die
-            qq{The "name" parameter when creating a parameter validator must be a scalar, you passed $type\n};
+        croak
+            qq{The "name" parameter when creating a parameter validator must be a scalar, you passed $type};
     }
 
     my @unknown = sort grep { !$known{$_} } keys %p;
     if (@unknown) {
-        die
-            "You passed unknown parameters when creating a parameter validator: [@unknown]\n";
+        croak
+            "You passed unknown parameters when creating a parameter validator: [@unknown]";
     }
 
     my $self = bless \%p, $class;
@@ -96,6 +117,8 @@ sub _source { $_[0]->{_source} }
 
 sub _env { $_[0]->{_env} }
 
+sub named_to_list { $_[0]->{named_to_list} }
+
 sub subref {
     my $self = shift;
 
@@ -133,16 +156,50 @@ sub _compile {
         $self->_compile_named_args_check;
     }
     elsif ( ref $self->params eq 'ARRAY' ) {
-        $self->_compile_positional_args_check;
+        if ( $self->named_to_list ) {
+            $self->_compile_named_args_list_check;
+        }
+        else {
+            $self->_compile_positional_args_check;
+        }
     }
 }
 
 sub _compile_named_args_check {
     my $self = shift;
 
-    push @{ $self->_source }, $self->_set_named_args_hash;
+    $self->_compile_named_args_check_body( $self->params );
+    push @{ $self->_source }, 'return %args;';
 
-    my $params = $self->params;
+    return;
+}
+
+sub _compile_named_args_list_check {
+    my $self = shift;
+
+    $self->_compile_named_args_check_body( { @{ $self->params } } );
+
+    my @keys = map { B::perlstring($_) } pairkeys @{ $self->params };
+
+    # If we don't handle the one-key case specially we end up getting a
+    # warning like "Scalar value @args{"bar"} better written as $args{"bar"}
+    # at ..."
+    if ( @keys == 1 ) {
+        push @{ $self->_source }, "return \$args{$keys[0]};";
+    }
+    else {
+        my $keys_str = join q{, }, @keys;
+        push @{ $self->_source }, "return \@args{$keys_str};";
+    }
+
+    return;
+}
+
+sub _compile_named_args_check_body {
+    my $self   = shift;
+    my $params = shift;
+
+    push @{ $self->_source }, $self->_set_named_args_hash;
 
     for my $name ( sort keys %{$params} ) {
         my $spec = $params->{$name};
@@ -168,15 +225,13 @@ sub _compile_named_args_check {
     }
 
     if ( $self->slurpy ) {
-        $self->_add_check_for_extra_hash_param_types( $self->slurpy )
+        $self->_add_check_for_extra_hash_param_types( $self->slurpy, $params )
             if ref $self->slurpy;
     }
     else {
-        $self->_add_check_for_extra_hash_params;
+        $self->_add_check_for_extra_hash_params($params);
     }
 
-    push @{ $self->_source }, 'return %args;';
-
     return;
 }
 
@@ -252,11 +307,12 @@ EOF
 }
 
 sub _add_check_for_extra_hash_param_types {
-    my $self = shift;
-    my $type = shift;
+    my $self   = shift;
+    my $type   = shift;
+    my $params = shift;
 
     $self->_env->{'%known'}
-        = { map { $_ => 1 } keys %{ $self->params } };
+        = { map { $_ => 1 } keys %{$params} };
 
     # We need to set the name argument to something that won't conflict with
     # names someone would actually use for a parameter.
@@ -275,10 +331,11 @@ EOF
 }
 
 sub _add_check_for_extra_hash_params {
-    my $self = shift;
+    my $self   = shift;
+    my $params = shift;
 
     $self->_env->{'%known'}
-        = { map { $_ => 1 } keys %{ $self->params } };
+        = { map { $_ => 1 } keys %{$params} };
     push @{ $self->_source }, <<'EOF';
 my @extra = grep { ! $known{$_} } keys %args;
 if ( @extra ) {
@@ -356,7 +413,7 @@ sub _munge_and_check_positional_params {
             $in_optional = 1;
         }
         elsif ($in_optional) {
-            die
+            croak
                 'Parameter list contains an optional parameter followed by a required parameter.';
         }
 
@@ -433,7 +490,7 @@ sub _add_default_assignment {
     my $name    = shift;
     my $default = shift;
 
-    die 'Default must be either a plain scalar or a subroutine reference'
+    croak 'Default must be either a plain scalar or a subroutine reference'
         if ref $default && reftype($default) ne 'CODE';
 
     my $qname = B::perlstring($name);
@@ -470,7 +527,7 @@ sub _add_type_check {
     my $spec   = shift;
 
     my $type = $spec->{type};
-    die "Passed a type that is not an object for $name: $type"
+    croak "Passed a type that is not an object for $name: $type"
         unless blessed $type;
 
     push @{ $self->_source }, sprintf( 'if ( exists %s ) {', $access )
@@ -502,7 +559,7 @@ sub _type_check {
         # Moose
         : $type->can('can_be_inlined')
         ? $self->_add_moose_check( $access, $name, $type )
-        : die 'Unknown type object ' . ref $type;
+        : croak 'Unknown type object ' . ref $type;
 }
 
 sub _add_type_tiny_check {
@@ -676,7 +733,7 @@ Params::ValidationCompiler::Compiler - Object that implements the check subrouti
 
 =head1 VERSION
 
-version 0.13
+version 0.14
 
 =for Pod::Coverage .*
 
